;==========================================================================
        Title  "Bike Advance"
;      http://www.transmic.net/
;      20 Decembre 2011      Version 6.50  for 16f628
;      16f628 Horloge interne 4MHz
;      cablage compatible cdi 16f628 PCM6.5
;      Copyright  Thierry PERCIN
;==========================================================================
; 21/05/06 Ajout switch pour choix limitation RPM.
; 27/07/06 Corrections: test led s'eteint a 29Hz et rpmlimit agit a 29Hz.
; 20/10/06 Augmentation du delay a low_rpm (periode x8 au lieu de x2)- Ajout ReverseOutput
; 20/06/07 Deplacement de la table en fin de programme.
; 05/07/07 Ajout 2em table et deplacement en fin de programme.(Utilisation du switch RpmLimit)
; 29/02/08 Ajout clignotement LED au demarrage
; 15/12/11 Ajout d'une shift LED, correction fin de map et taille map2. Version dans .HEX
; 09-01-13 For Dawid Ploskonka Counter:22 a 233, Prescaler:256, Delay:10s LowRpm:
;
;
;
; (TODO: modifier le delay_lowrpm en fonction du prescaler)
;==========================================================================
;
;       PIC 16F628 wiring
;
;==========================================================================
                  
;  Shift LED<----------- RA2   1+--O--+18  RA1
;  Choke relais<-------- RA3   2|     |17  RA0
;                        RA4   3|     |16  Osc
;  +5Vdc------RRRR------ MCLR  4|     |15  Osc
;  Ground--------------- Vss   5|     |14  Vcc ------------ +5Vdc
;                        RB0   6|     |13  RB7 <----------- switch (noRpmLimit)
;  ReverseOutput<------- RB1   7|     |12  RB6 -----------> Led
;  Thyristor<----------- RB2   8|     |11  RB5 <----------- switch (map selection)
;                        RB3   9+-----+10  RB4 <----------- pickup36

;==========================================================================
;
;       Directives
;
;==========================================================================
; mplab settings

;	ERRORLEVEL -224		; suppress annoying message because of option/tris
	ERRORLEVEL -302		; suppress message because of bank select in setup ports
         LIST   P=PIC16F628    	; processor type
;==============================================================================
;
        #include "P16F628.INC"  ; Include header file

     __config  _CP_OFF & _INTRC_OSC_NOCLKOUT & _LVP_OFF & _WDT_OFF & _PWRTE_ON & _BODEN_ON & _MCLRE_OFF

; '__CONFIG' prcise les paramtres encods dans le processeur au moment de
; la programmation du processeur. Les dfinitions sont dans le fichier include.
; Choisir une valeur par groupe.

; Protection du programme
; -----------------------
;_CP_ALL        protection totale
;_CP_75         protection de 1000  1FFF
;_CP_50         protection de 1F00  1FFF
;_CP_OFF        Pas de protection

; Protection de la EEprom
; -----------------------
;_DATA_CP_ON    Mmoire EEprom protge
;_DATA_CP_OFF   Mmoire EEprom dprotge

; Programmation sur circuit
; -------------------------
;_LVP_ON        RB3 permet la programmation srie de la PIC
;_LVP_OFF       RB3 en utilisation normale

; Reset de la PIC si tension <4V
; ------------------------------
; _BODEN_ON     Reset tension en service
;               Valide PWRTE_ON automatiquement
; _BODEN_OFF    Reset tension hors service

;Utilisation de la pin MCLR
; ------------------------------
;_MCLRE_ON      RA5/MCLR est utilise pour le reset
;_MCLRE_OFF     RA5/MCLR est utilise comme une entree/sortie,  MCLR internally tied to Vcc

; Retard  la mise sous tension
; -----------------------------
;_PWRTE_OFF     Dmarrage rapide
;_PWRTE_ON      Dmarrage temporis

; Watchdog
; --------
;_WDT_ON        Watchdog en service
;_WDT_OFF       Watchdog hors service

; Oscillateur
; -----------
;_ER_OSC_CLKOUT         clckout sur RA6 resistance externe surRA7             
;_ER_OSC_NOCLKOUT       I/O sur RA6 resistance externe sur RA7
;                       (1K*10MHz-20K*5.3MHz-100k*1.6MHz-220M*800KHz-1M*200KHz)   
;_INTRC_OSC_CLKOUT      oscillateur interne clkout sur RA6 I/O sur RA7 (4MHz)    
;_INTRC_OSC_NOCLKOUT    oscillateur interne I/O sur RA6 et RA7 (4MHz)        
;_EXTCLK_OSC            signal clock fournit au pic sur RA7 I/O sur RA6                
;_LP_OSC                Oscillateur basse vitesse (?<F<200Khz)        
;_XT_OSC                Oscilateur moyenne vitesse (0,1MHz<F<4Mhz)     
;_HS_OSC                Oscillateur haute vitesse (1Mhz<F<20Mhz)
    
; Page 0: 040-139 page1: 140-239 page2: 240-339 page3: 340-439
; Page 4: 440-539 page5: 540-639 page6: 640-739 page7: 740-839
;*****************************************************************************
;                               ASSIGNATIONS SYSTEME                         *
;*****************************************************************************

; REGISTRE OPTION_REG (configuration) (banque 1)
; -----------------------------------
;	   n bits:  '76543210'
OPTIONVAL    EQU    B'10000111'		; prescaler = 256
	
	    ; RBPU      b7 : 1= Rsistance rappel +5V hors service
            ; INTEDG    b6 : 1= Interrupt sur front montant de RB0
            ;                0= Interrupt sur front descendant de RB0
            ; TOCS      b5 : 1= source clock = transition sur RA4
            ;                0= horloge interne
            ; TOSE      b4 : 1= Slection front descendant RA4(si B5=1)
            ;                0= Slection front montant RA4
            ; PSA       b3 : 1= Assignation prdiviseur sur Watchdog
            ;                0= Assignation prdiviseur sur Tmr0
            ; PS2/PS0   b2/b0 valeur du prdiviseur
            ;           000 =  1/1 (watchdog) ou 1/2 (tmr0)
            ;           001 =  1/2               1/4
            ;           010 =  1/4               1/8
            ;           011 =  1/8               1/16
            ;           100 =  1/16              1/32
            ;           101 =  1/32              1/64
            ;           110 =  1/64              1/128
            ;           111 =  1/128             1/256

; REGISTRE INTCON (contrle interruptions standard) (banque 0)
; -------------------------------------------------
;	   n bits:  '76543210'
INTCONVAL    EQU    B'11100000'
	    ; GIE       b7 : 1= INT GLOBALE autorisee.
            ; PEIE      b6 : 1= INT PERIPHERIQUES autorises. (timer1 en fait partie)
            ; T0IE      b5 : 1= INT par debordement Timer0 autorises.
            ; INTE      b4 : 0= INT de RB0/INT interdites. (INTERRUPTIONS EXTERNES)
            ; RBIE      b3 : 0= INT du port RB4  RB7 interdites.
            ; T0IF      b2 : flag mis  1 par dbordement du Timer0
            ; INTF      b1 : flag  mis  1 par une INT provoque par la ligne RB0/INT du port B
            ; RBIF      b0 : flag mis  1 si changement d'tat des entres RB4  RB7 du port B. attention RB1  RB3 -> pas d'INT

; REGISTRE PIE1 (contrle interruptions priphriques)
; ----------------------------------------------------
;	     n bits:  '76543210'
PIE1VAL        EQU    B'00000000'
            ; EEIE      b7 : masque interrupt criture EEPROM
            ; CMIE      b6 : masque interrupt comparateur
            ; RCIE      b5 : masque interrupt rception USART
            ; TXIE      b4 : masque interrupt transmission USART
            ; RESERVED  b3 : rserv, laisser  0
            ; CCP1IE    b2 : masque interrupt CCP1
            ; TMR2IE    b1 : masque interrupt TMR2 = PR2
            ; TMR1IE    b0 : masque interrupt dbordement tmr1

; REGISTRE CMCON (COMPARATEURS)
; ------------------------------------
;	  n bits:  '76543210'
CMCONVAL    EQU    B'00000111' 
            ; C2OUT     b7 :sortie comparateur2
            ; C1OUT     b6 :sortie comparateur1
            ; C2INV     b5 :inverseur comparateur2
            ; C1INV     b4 :inverseur comparateur1
            ; CIS       b3 :selection entree des comparateurs
            ; CM2-CM0   b2-0 :mode des comparateurs
            ; configurer le mode 111 pour utiliser RA0 RA1 RA2 RA3 en I/O

; set comparators off, PORT A en E/S numriques 

; REGISTRE VRCON (voltage reference module)
; ------------------------------------
;	  n bits:  '76543210'
VRCONVAL    EQU    B'00000000'
            ; VREN      b7 :Validation du module
            ; VROE      b6 :Sortie sur RA2
            ; VRR       b5 :Choix de la plage 
            ; RESERVED  b4 : rserv, laisser  0
            ; VR3-VR0   b3-0 :Choix de la valeur dans la plage 
            ; si VRR=1 VREF=(VR[3:0]/24)*Vdd
            ; si VRR=0 VREF=Vdd/4(VR[3:0]/32)*Vdd

; DIRECTION DES PORTS I/O
; -----------------------
;	  n bits:  '76543210'
DIRPORTA    EQU    B'00000000'      ; Direction PORTA (1=entre)
DIRPORTB    EQU    B'10110000'      ; Direction PORTB (1=entre)
                                    ;  switch (rpm limit) --------> RB7
                                    ;  switch (map select) -------> RB5
                                    ;  pickup 36        ---------> RB4

;*****************************************************************************
;                           ASSIGNATIONS D'ADRESSE PROGRAMME                           *
;*****************************************************************************
;
; kickstart at 300rpm|periode=200ms|5Hz
; perh_l=200000(s)/4(s)=50000
; perl=50000-(195*256)=80 perh=195
; perh / 2 (because line 436)= 97

version     EQU D'65'           ; Version number, just for information
                                ;

;*****************************************************************************
;                                  DEFINE                                    *
;*****************************************************************************

#DEFINE sled		PORTA,1	; [pin01]= output to shift led
#DEFINE choke		PORTA,3	; [pin03]= output to choke relay
#DEFINE revcoil		PORTB,1	; [pin07]= reverse output to coil
#DEFINE coil		PORTB,2	; [pin08]= output to coil
#DEFINE pickup		PORTB,4	; [pin10]= sensor input
#DEFINE selectmap	PORTB,5	; [pin11]= input select map1 or map2
#DEFINE led		PORTB,6	; [pin12]= output to led
#DEFINE noRpmLimit	PORTB,7	; [pin13]= spark even if RPM are too high

#DEFINE firstdelay	0x62		; Bit de sortie de la led verte 


;*****************************************************************************
;                             MACROS                                         *
;*****************************************************************************

; Changements de banques

BANK0	macro			; passer en banque0
	bcf	STATUS,RP0
	bcf	STATUS,RP1
	endm

BANK1	macro			; passer en banque1
	bsf	STATUS,RP0
	bcf	STATUS,RP1
	endm

BANK2	macro			; passer en banque2
	bcf	STATUS,RP0
	bsf	STATUS,RP1
	endm

BANK3	macro			; passer en banque3
	bsf	STATUS,RP0
	bsf	STATUS,RP1
	endm

;*****************************************************************************
;                      VARIABLES ZONE COMMUNE                                *
;*****************************************************************************
; Zone de 16 bytes
; ----------------

    CBLOCK  0x70                ; Dbut de la zone (0x70  0x7F)
        w_temp : 1              ; Sauvegarde registre W
        status_temp : 1         ; sauvegarde registre STATUS
    ENDC

;*****************************************************************************
;                        VARIABLES BANQUE 0                                  *
;*****************************************************************************
; Zone de 80 bytes
; ----------------

    CBLOCK  0x20             ; Dbut de la zone (0x20  0x6F)
        tmr0h  : 1           ; Timer register high
        perl   : 1           ; periode register low
        perh   : 1           ; periode register high
        retard : 1           ; delay before spark in 4us units
        cmpt1  : 1           ; compteur pour tempo 1ms
	cmpt2  : 1           ; compteur pour clignotement led
        cmpt3  : 1           ; nombre de clignotements led
    ENDC                     ; Fin de la zone                        

var1    EQU H'006E'          ; adresse impose

;*****************************************************************************
;                        VARIABLES BANQUE 1                                  *
;*****************************************************************************
; Zone de 80 bytes
; ----------------

    CBLOCK  0xA0                ; Dbut de la zone (0xA0  0xEF)
                                ; non utilisee
    ENDC                        ; Fin de la zone                        

;*****************************************************************************
;                        VARIABLES BANQUE 2                                  *
;*****************************************************************************
; Zone de 48 bytes
; ----------------

    CBLOCK  0x120               ; Dbut de la zone (0x120  0x14F)
                                ; non utilisee
    ENDC                        ; Fin de la zone                        

;==========================================================================
;
;       Main Program
;
;==========================================================================

	ORG     0x000			; Adresse de dpart aprs reset

	movlw   version			; Version Number ( just for displaying on Hex )
	GOTO    init
	
;==========================================================================
; TRAITEMENT DES INTERRUPTIONS (Aiguillage vers routines)
;==========================================================================
; Si on n'utilise pas l'adressage indirect dans les interrupts, on se passera
; de sauvegarder FSR

;RAPPEL Un seul vecteur d'interruption, d'adresse 004 est diponible
;Quelle que soit la cause de l'interruption, le PC est donc charg par 004
; vous ensuite de tester les diffrents indicateurs pour savoir quel est la source de l'INT.
;voir le registre INTCON
	
;-- sauvegarder registres    
	ORG     0x004               ; adresse d'interruption
	movwf   w_temp              ; sauver registre W
	swapf   STATUS,w            ; swap status avec rsultat dans w
	movwf   status_temp         ; sauver status swapp

	; Interruption TMR0 <=> 14 cycles en tout ( 12 + clr TMR0 )
	; -----------------
	; Traitement de l'interruption qd TMR0 passe de 255 a 0. Avec Q=4MHz:
	; tout les 256cycles * 128prescaler * 1s = 32768s =18rpm
	; tout les 256cycles * 256prescaler * 1s = 65536s = 9rpm

	INCF    tmr0h,f             ; tmr0_high + 1
	BCF     INTCON,T0IF         ; reset du flag d'interruption

;-- restaurer registres
	swapf   status_temp,w       ; swap ancien status, rsultat dans w
	movwf   STATUS              ; restaurer status
	swapf   w_temp,f            ; Inversion L et H de l'ancien W & sans modifier Z
	swapf   w_temp,w            ; Rinversion de L et H dans W & W restaur sans modifier status
	retfie                      ; return from interrupt

;*****************************************************************************
;                          INITIALISATIONS                                   *
;*****************************************************************************
init
            ; initialisation PORTS (banque 0 et 1)
            ; ------------------------------------

	BANK0                   ; slectionner banque0        
	CLRF   PORTA           	; Initialize Sorties PORTA  0
	CLRF   PORTB           	; Initialize Sorties PORTB  0

	BANK1                   ; passer en banque1
	movlw   DIRPORTA        ; Direction PORTA
	movwf   TRISA           ; criture dans registre direction
	movlw   DIRPORTB        ; Direction PORTB
	movwf   TRISB           ; criture dans registre direction

            ; Registre d'options (banque 1)
            ; -----------------------------
	movlw   OPTIONVAL           ; charger masque
	movwf   OPTION_REG          ; initialiser registre option

            ; registres interruptions (banque 1 et 0)
            ; ----------------------------------
	movlw   INTCONVAL           ; charger valeur registre interruption
	movwf   INTCON              ; initialiser interruptions
	movlw   PIE1VAL             ; Initialiser registre 
	movwf   PIE1                ; interruptions priphriques 1
	movlw   VRCONVAL            ; charger valeur registre interruption
	movwf   VRCON               ; initialiser interruptions
	BANK0                       ; slectionner banque 0
	movlw   CMCONVAL            ; charger valeur registre interruption
	movwf   CMCON               ; initialiser interruptions

            ; autoriser interruptions (banque 0)
            ; ----------------------------------
	clrf    PIR1                ; effacer flags 1
	bsf     INTCON,GIE          ; valider interruptions

;---------------------------------------------------------------------
; Delai a la toute premiere etincelle
	CLRF    TMR0		; reset the time conter
	MOVLW firstdelay	; and define perh = 97 for the
	MOVWF tmr0h		; 1st delay map 300rpm

;---------------------------------------------------------------------
; Starter actif jusqu'a 5000rpm
	BSF 	choke

;---------------------------------------------------------------------
; Clignotement led (4x2+1=9 x 255ms = 2.3sec)
	MOVLW 4
	MOVWF cmpt3		; 4 clignotements
;	goto    $ + 0B		; Decommenter pour accelerer PIC Simulator IDE
blink_led
	BSF     led		; led on
	call tempo		; 255 x 1ms temporisation
	DECFSZ  cmpt2,f		; dcrmenter compteur2
	GOTO    blink_led	; si pas 0, boucler
	BCF     led		; led off
	call tempo		; 255 x 1ms temporisation
	DECFSZ  cmpt2,f		; dcrmenter compteur2
	GOTO    $ - 2		; si pas 0, boucler
	DECFSZ  cmpt3,f		; dcrmenter compteur2
	GOTO    blink_led	; si pas 0, boucler

;---------------------------------------------------------------------
; Note: Delay are calculated in multiples of 9 microseconds ("waiting sub-routine")
; First value is for 6510 RPM (TMR0 count=36) and last value is for 1953 RPM (TMR0 count=120)
; First value is for 6510 RPM (TMR0 count=36) and last value is for 1723 RPM (TMR0 count=136)
; 4 strokes with one wasted  : 1cyl|6510rpm=108.5Hz|9.2ms   [9200/256(prescaler)= 36] <--
; spark or 2 strokes         : 1cyl|1723rpm= 28.7Hz|34.8ms [34800/256(prescaler)= 136]<--
;	

pickup_high
	BTFSS   pickup	; pickup=1?
	GOTO    pickup_low	; no
	BSF     led		; yes, led on (even if hors map)
	GOTO    pickup_high	; test input


pickup_low 
	BCF     led		; pickup low then led off
	BTFSS   pickup		; pickup=1?
	GOTO    pickup_low	; no

				; yes, memorize the time past
	MOVF    TMR0,W		; perl=TMR0
	MOVWF   perl
	MOVF    tmr0h,W		; perh=tmr0h;
	MOVWF   perh
	CLRF    TMR0		; and we reset the time conters
	CLRF    tmr0h
	BSF	led		; pickup high then led on, even if hors map

	;if ((!perh)&&(perl>22)&&(perl<233))
	; ---- too low rpm
	MOVF    perh,f
	BTFSS   STATUS,Z	; perh >0? (very low rpm)
	GOTO    hors_map	; yes

	; ---- too high rpm
	MOVF    perl,W		; no, perl <23?  (>10653rpm?)
	SUBLW   0x16		; 22 * 256prescaler * 1s = 5.64ms = 177Hz = 10653rpm
	BTFSC   STATUS,C	; rpm = 60.000.000 / (perl*256)
	GOTO    hors_map	; yes

	BSF	led		; pickup high then led on, but off if hors map

	; ----- choke off
	MOVF    perl,W		; no, perl <58? (>4040rpm?)
	SUBLW   0x3A		;	58 * 256(prescaler) * 1s = 14848s = 67Hz => 67*60 = 4040rpm
	BTFSC   STATUS,C	;	perl = 60.000.000 / (rpm*256)
	BCF     choke		; yes: starter off

	; ----- shift led
	MOVF    perl,W		; no, perl =47? (4882 a 4980rpm?)
	SUBLW   0x2F		; 	47 * 256(prescaler) * 1s = 12032s = 83Hz => 83*60 = 4980rpm
	BTFSS STATUS,Z
	GOTO $ + 2
	BSF     sled		; yes: led on
	
	; ---- low rpm
	MOVF    perl,W		; no, perl >232? (<1005rpm?)
	SUBLW   0xEA		; 234 * 256prescaler * 1s = 62.50ms = 16Hz = 1005rpm
	BTFSS   STATUS,C
	GOTO    hors_map	; yes

	; ---- rpm OK, read map to get a delay
	MOVLW   0x17      		; no, perl=perl-23;
	SUBWF   perl,f
	MOVF    perl,W

	BTFSC	selectmap	; is table select low? (jumper on)
	CALL    map2		; no, read the second advance
	BTFSS	selectmap	; yes, is table select high? (jumper off)
	CALL    map1		; no, read the first advance

	MOVWF   retard

;      	BTFSC   noRpmLimit	; correction switch on?
;	GOTO    decr_ret	; input=1 then no correction
;	MOVLW	5		; yes, input=0 then lower advance -2deg/2100rpm
;	ADDWF	retard,f

	; ---- delay before spark
	; This sub-routine wait:
	; 10 cycles of 1s = 10s x retard (from 10s to 2550ms at 4MHZ)

decr_ret
	NOP			; decfsz &retard,f	1cycle
	NOP			;			1cycle
	NOP			;			1cycle
	NOP			;			1cycle
	NOP			;			1cycle
	NOP			;			1cycle
	NOP			;			1cycle
	DECFSZ  retard,f	; retard =0 ?		1cycle
	GOTO    decr_ret	; no			2cycle

	; ---- spark
	BSF	coil		; yes, thyristor on
        BCF	revcoil
	call tempo		; 1ms temporisation
	BCF     coil		; thyristor off
        BSF	revcoil
        
        ; --- return and wait for next pickup pulse
	GOTO    pickup_high	; test input

;----------------------------------------------------------------------
hors_map
	MOVF    perh,f
	BTFSS   STATUS,Z	; perh >0?
	GOTO    low_rpm		; yes

	MOVF    perl,W		; no. perl>232? (<1005rpm - <16Hz?)
	SUBLW   0xEA
	BTFSC   STATUS,C
	GOTO    high_rpm	; no, test if no more spark or RPM unlimited

;----------------------------------------------------------------------

; From 0 to 1005 RPM

; If use of periode * 2 and prescaler = 256
; delay = [ (TMR0 x 2) x 4us + (tmr0h x 2) x 1ms ]
; 1740rpm = 34500us. TMR0 = 34500/128=135 TMR0 = 135, tmr0h = 0
; delay = [ (135x2) x 4us + (0 x 2) x 1ms ]
; delay = [ (270 x 4us) + (0ms) ]= 1080us = 1.08ms = 11
;
; 600rpm = 100000us. TMR0 = 100000/256=390 => TMR0 = 390-255=135, tmr0h = 1
; delay = [ (135x2) x 4us + (1 x 2) x 1ms ]
; delay = [ (270 x 4us) + ( 2 x 1ms) ]= 3080us = 3.08ms = 11

; If use of periode * 2 and prescaler = 128
; delay = [ (TMR0 x 2) x 4us + (tmr0h x 2) x 1ms ]
; 1740rpm = 34500us. TMR0 = 34500/128=270 TMR0 = 270-255=15, tmr0h = 1
; delay = [ (15x2) x 4us + (1 x 2) x 1ms ] = 2120us = 2.12ms = 22
;
; 600rpm = 100000us. TMR0 = 100000/128=781 => TMR0 = 781-255-255-255=16, tmr0h = 3
; delay = [ (16x2) x 4us + (3 x 2) x 1ms ] = 6128us = 6.128ms = 22

; If use of periode * 4
; delay = [ (TMR0 x 4) x 4us + (tmr0h x 4) x 1ms ]
; 1740rpm = 34500us. TMR0 = 34500/128=270 TMR0 = 270-255=15, tmr0h = 1
; delay = [ (15x4) x 4us + (1 x 4) x 1ms ]
; delay = [ (60 x 4us) + (4ms) ]= 4240us = 4.24ms = 44
;
; 600rpm = 100000us. TMR0 = 100000/128=781 => TMR0 = 781-255-255-255=16, tmr0h = 3
; delay = [ (16x4) x 4us + (3 x 4) x 1ms ]
; delay = [ (64 x 4us) + ( 12 x 1ms) ]= 12256us = 12.256ms = 44

; If use of periode * 4   +1
; delay = [ (TMR0 x 4)+1  x 4us + (tmr0h x 4)+1 x 1ms ]
; 1740rpm = 34500us. TMR0 = 34500/128=270 TMR0 = 270-255=15, tmr0h = 1
; delay = [ (15x4)+1 x 4us + (1 x 4)+1 x 1ms ]
; delay = [ (60 x 4us) + (5ms) ]= 5240us = 5.24ms = 54,6
;
; 600rpm = 100000us. TMR0 = 100000/128=781 => TMR0 = 781-255-255-255=16, tmr0h = 3
; delay = [ (16x4)+1 x 4us + (3x4)+1 x 1ms ]
; delay = [ (64 x 4us) + ( 13 x 1ms) ]= 13256us = 13.26ms = 47,7

; Presc 128 | 256
; ----------|-----
; *2    22 | 11
; *4    44 | 22
; *8    88 | 44

low_rpm
	BCF     STATUS,C	; yes perl>232 (<1005rpm, <16Hz )
	RLF     perl,f		; periode *2
	BTFSC	STATUS,C	; si >255
	INCF	perh,f		; report de la retenue
	RLF     perh,f
;
	RLF     perl,f		; periode *4
	BTFSC	STATUS,C
	INCF	perh,f
	RLF     perh,f
;
;	RLF     perl,f		; periode *8
;	BTFSC	STATUS,C
;	INCF	perh,f
;	RLF     perh,f

	INCF    perh,f		; + 1
;	INCF    perh,f		; + 1
;	INCF    perh,f		; + 1
;	MOVLW	1		; w=1
;	SUBWF	perh,f		; -1

;---------------------------------------------------------------------
; This sub-routine wait:
; (4 s x perl) + [perlh x (4 s x 256)] = (from 4s to 262ms at 4MHZ)

delay_lowrpm
;	goto    $+6		; Decommenter pour accelerer PIC Simulator IDE
	NOP			;                 	1cycle
	DECFSZ  perl,f		; perl=0 ?		1cycle
	GOTO    delay_lowrpm	; no			2cycle
	DECFSZ  perh,f		; perh=0 ?		1cycle
	GOTO    delay_lowrpm	; no			2cycle

	; ---- spark
	BSF	coil		; yes, thyristor on
	BCF	revcoil
	call tempo		; 1ms temporisation
	BCF     coil		; thyristor off
	BSF	revcoil
	; --- return and wait for next pickup pulse
	GOTO    pickup_high	; test input

;----------------------------------------------------------------------
; Above 10653 RPM

high_rpm
	BCF     STATUS,C	; yes (perl>233) | >10653rpm)
	BTFSC   noRpmLimit	; switch on (input=0)?
	GOTO    pickup_high	; no,  RPM too high and RpmLimit=on    = no more spark

	; ---- spark
	BSF	coil		; yes, RPM too high but RpmLimit=off than still sparkling
	BCF	revcoil
	call tempo		; 1ms temporisation
	BCF     coil		; thyristor off
	BSF	revcoil
	; --- return and wait for next pickup pulse
	GOTO    pickup_high	; test input

;---------------------------------------------------------------------
; Cette sous-routine introduit un retard de:
; 255boucles x 4 cycles de 1s = 255 * 4 s = 1ms a 4MHZ
;---------------------------------------------------------------------
tempo
	clrf	cmpt1		; effacer compteur1
;	movlw   0x02            ; Decommenter pour accelerer PIC Simulator IDE
;	movwf   cmpt1           ; Decommenter pour accelerer PIC Simulator IDE
	nop			; perdre 1 cycle		1cycle
	decfsz	cmpt1, f	; dcrmenter compteur1		1cycle
	goto	$ -2		; si pas 0, boucler		2cycles
	return			; retour de la sous-routine

;----------------------------------------------------------------------
;    MAPS
;---------------------------------------------------------------------

	ORG 0x200
map1
	BCF    PCLATH,2
	BSF    PCLATH,1
	BCF     PCLATH,0	; select page at b'010' for ORG 0x200
        ADDWF   PCL,f		;line 657
        retlw 0x30         ;10653rpm | 5 | 0,48ms 
        retlw 0x32         ;10190rpm | 5 | 0,51ms 
        retlw 0x34         ;9766rpm | 5 | 0,53ms 
        retlw 0x37         ;9375rpm | 5 | 0,55ms 
        retlw 0x01         ;9014rpm | 36 | 0ms 
        retlw 0x01         ;8681rpm | 36 | 0ms 
        retlw 0x01         ;8371rpm | 36 | 0ms 
        retlw 0x01         ;8082rpm | 36 | 0ms 
        retlw 0x01         ;7813rpm | 36 | 0ms 
        retlw 0x01         ;7560rpm | 36 | 0ms 
        retlw 0x01         ;7324rpm | 36 | 0ms 
        retlw 0x01         ;7102rpm | 36 | 0ms 
        retlw 0x01         ;6893rpm | 36 | 0ms 
        retlw 0x01         ;6696rpm | 36 | 0ms 
        retlw 0x01         ;6510rpm | 36 | 0ms 
        retlw 0x01         ;6334rpm | 36 | 0ms 
        retlw 0x01         ;6168rpm | 36 | 0ms 
        retlw 0x01         ;6010rpm | 36 | 0ms 
        retlw 0x01         ;5859rpm | 36 | 0ms 
        retlw 0x01         ;5716rpm | 36 | 0ms 
        retlw 0x01         ;5580rpm | 36 | 0ms 
        retlw 0x01         ;5451rpm | 36 | 0ms 
        retlw 0x01         ;5327rpm | 36 | 0ms 
        retlw 0x01         ;5208rpm | 36 | 0ms 
        retlw 0x01         ;5095rpm | 36 | 0ms 
        retlw 0x01         ;4987rpm | 36 | 0ms 
        retlw 0x01         ;4883rpm | 36 | 0ms 
        retlw 0x01         ;4783rpm | 36 | 0ms 
        retlw 0x01         ;4688rpm | 36 | 0ms 
        retlw 0x01         ;4596rpm | 36 | 0ms 
        retlw 0x01         ;4507rpm | 36 | 0ms 
        retlw 0x01         ;4422rpm | 36 | 0ms 
        retlw 0x01         ;4340rpm | 36 | 0ms 
        retlw 0x01         ;4261rpm | 36 | 0ms 
        retlw 0x01         ;4185rpm | 36 | 0ms 
        retlw 0x01         ;4112rpm | 36 | 0ms 
        retlw 0x01         ;4041rpm | 36 | 0ms 
        retlw 0x01         ;3972rpm | 36 | 0ms 
        retlw 0x01         ;3906rpm | 36 | 0ms 
        retlw 0x01         ;3842rpm | 36 | 0ms 
        retlw 0x01         ;3780rpm | 36 | 0ms 
        retlw 0x01         ;3720rpm | 36 | 0ms 
        retlw 0x01         ;3662rpm | 36 | 0ms 
        retlw 0x01         ;3606rpm | 36 | 0ms 
        retlw 0x01         ;3551rpm | 36 | 0ms 
        retlw 0x01         ;3498rpm | 36 | 0ms 
        retlw 0x01         ;3447rpm | 36 | 0ms 
        retlw 0x01         ;3397rpm | 36 | 0ms 
        retlw 0x01         ;3348rpm | 36 | 0ms 
        retlw 0x01         ;3301rpm | 36 | 0ms 
        retlw 0x01         ;3255rpm | 36 | 0ms 
        retlw 0x01         ;3211rpm | 36 | 0ms 
        retlw 0x15         ;3167rpm | 32 | 0,21ms 
        retlw 0x2A         ;3125rpm | 28 | 0,43ms 
        retlw 0x2B         ;3084rpm | 28 | 0,43ms 
        retlw 0x2B         ;3044rpm | 28 | 0,44ms 
        retlw 0x37         ;3005rpm | 26 | 0,55ms 
        retlw 0x38         ;2967rpm | 26 | 0,56ms 
        retlw 0x38         ;2930rpm | 26 | 0,57ms 
        retlw 0x39         ;2894rpm | 26 | 0,58ms 
        retlw 0x3A         ;2858rpm | 26 | 0,58ms 
        retlw 0x46         ;2824rpm | 24 | 0,71ms 
        retlw 0x47         ;2790rpm | 24 | 0,72ms 
        retlw 0x48         ;2757rpm | 24 | 0,73ms 
        retlw 0x49         ;2725rpm | 24 | 0,73ms 
        retlw 0x4A         ;2694rpm | 24 | 0,74ms 
        retlw 0x4B         ;2663rpm | 24 | 0,75ms 
        retlw 0x4B         ;2633rpm | 24 | 0,76ms 
        retlw 0x4C         ;2604rpm | 24 | 0,77ms 
        retlw 0x5A         ;2576rpm | 22 | 0,91ms 
        retlw 0x5B         ;2548rpm | 22 | 0,92ms 
        retlw 0x5C         ;2520rpm | 22 | 0,93ms 
        retlw 0x5D         ;2493rpm | 22 | 0,94ms 
        retlw 0x5E         ;2467rpm | 22 | 0,95ms 
        retlw 0x7A         ;2441rpm | 18 | 1,23ms 
        retlw 0x7C         ;2416rpm | 18 | 1,24ms 
        retlw 0x7D         ;2392rpm | 18 | 1,25ms 
        retlw 0x7E         ;2367rpm | 18 | 1,27ms 
        retlw 0x80         ;2344rpm | 18 | 1,28ms 
        retlw 0x81         ;2321rpm | 18 | 1,29ms 
        retlw 0x82         ;2298rpm | 18 | 1,31ms 
        retlw 0x99         ;2275rpm | 15 | 1,54ms 
        retlw 0x9B         ;2254rpm | 15 | 1,55ms 
        retlw 0x9C         ;2232rpm | 15 | 1,57ms 
        retlw 0x9E         ;2211rpm | 15 | 1,58ms 
        retlw 0x9F         ;2190rpm | 15 | 1,6ms 
        retlw 0xA1         ;2170rpm | 15 | 1,61ms 
        retlw 0xA2         ;2150rpm | 15 | 1,63ms 
        retlw 0xA4         ;2131rpm | 15 | 1,64ms 
        retlw 0xA5         ;2111rpm | 15 | 1,66ms 
        retlw 0xA7         ;2093rpm | 15 | 1,67ms 
        retlw 0xA8         ;2074rpm | 15 | 1,69ms 
        retlw 0xAA         ;2056rpm | 15 | 1,7ms 
        retlw 0xAB         ;2038rpm | 15 | 1,72ms 
        retlw 0xAD         ;2020rpm | 15 | 1,73ms 
        retlw 0xAE         ;2003rpm | 15 | 1,75ms 
        retlw 0xB0         ;1986rpm | 15 | 1,76ms 
        retlw 0xB1         ;1970rpm | 15 | 1,78ms 
        retlw 0xBB         ;1953rpm | 14 | 1,88ms 
        retlw 0xBD         ;1937rpm | 14 | 1,89ms 
        retlw 0xBE         ;1921rpm | 14 | 1,91ms 
        retlw 0xC0         ;1905rpm | 14 | 1,92ms 
        retlw 0xC1         ;1890rpm | 14 | 1,94ms 
        retlw 0xC3         ;1875rpm | 14 | 1,96ms 
        retlw 0xC5         ;1860rpm | 14 | 1,97ms 
        retlw 0xC6         ;1845rpm | 14 | 1,99ms 
        retlw 0xC8         ;1831rpm | 14 | 2ms 
        retlw 0xC9         ;1817rpm | 14 | 2,02ms 
        retlw 0xCB         ;1803rpm | 14 | 2,03ms 
        retlw 0xCC         ;1789rpm | 14 | 2,05ms 
        retlw 0xCE         ;1776rpm | 14 | 2,07ms 
        retlw 0xD0         ;1762rpm | 14 | 2,08ms 
        retlw 0xD1         ;1749rpm | 14 | 2,1ms 
        retlw 0xD3         ;1736rpm | 14 | 2,11ms 
        retlw 0xD4         ;1723rpm | 14 | 2,13ms 
        retlw 0xD6         ;1711rpm | 14 | 2,14ms 
        retlw 0xD7         ;1698rpm | 14 | 2,16ms 
        retlw 0xD9         ;1686rpm | 14 | 2,17ms 
        retlw 0xDB         ;1674rpm | 14 | 2,19ms 
        retlw 0xDC         ;1662rpm | 14 | 2,21ms 
        retlw 0xDE         ;1651rpm | 14 | 2,22ms 
        retlw 0xDF         ;1639rpm | 14 | 2,24ms 
        retlw 0xE1         ;1628rpm | 14 | 2,25ms 
        retlw 0xE2         ;1616rpm | 14 | 2,27ms 
        retlw 0xE4         ;1605rpm | 14 | 2,28ms 
        retlw 0xE5         ;1594rpm | 14 | 2,3ms 
        retlw 0xE7         ;1584rpm | 14 | 2,32ms 
        retlw 0xF3         ;1573rpm | 13 | 2,44ms 
        retlw 0xF5         ;1563rpm | 13 | 2,45ms 
        retlw 0xF6         ;1552rpm | 13 | 2,47ms 
        retlw 0xF8         ;1542rpm | 13 | 2,49ms 
        retlw 0xFA         ;1532rpm | 13 | 2,5ms 
        retlw 0xFB         ;1522rpm | 13 | 2,52ms 
        retlw 0xFD         ;1512rpm | 13 | 2,54ms 
        retlw 0xFF         ;1502rpm | 13 | 2,55ms 
        retlw 0xFF         ;1493rpm | 13 | 2,57ms 
        retlw 0xFF         ;1483rpm | 13 | 2,58ms 
        retlw 0xFF         ;1474rpm | 13 | 2,6ms 
        retlw 0xFF         ;1465rpm | 13 | 2,62ms 
        retlw 0xFF         ;1456rpm | 13 | 2,63ms 
        retlw 0xFF         ;1447rpm | 13 | 2,65ms 
        retlw 0xFF         ;1438rpm | 13 | 2,67ms 
        retlw 0xFF         ;1429rpm | 13 | 2,68ms 
        retlw 0xFF         ;1420rpm | 13 | 2,7ms 
        retlw 0xFF         ;1412rpm | 13 | 2,72ms 
        retlw 0xFF         ;1403rpm | 13 | 2,73ms 
        retlw 0xFF         ;1395rpm | 13 | 2,75ms 
        retlw 0xFF         ;1387rpm | 12 | 2,88ms 
        retlw 0xFF         ;1379rpm | 12 | 2,9ms 
        retlw 0xFF         ;1371rpm | 12 | 2,92ms 
        retlw 0xFF         ;1363rpm | 12 | 2,94ms 
        retlw 0xFF         ;1355rpm | 12 | 2,95ms 
        retlw 0xFF         ;1347rpm | 12 | 2,97ms 
        retlw 0xFF         ;1339rpm | 12 | 2,99ms 
        retlw 0xFF         ;1332rpm | 12 | 3ms 
        retlw 0xFF         ;1324rpm | 12 | 3,02ms 
        retlw 0xFF         ;1317rpm | 12 | 3,04ms 
        retlw 0xFF         ;1309rpm | 12 | 3,05ms 
        retlw 0xFF         ;1302rpm | 12 | 3,07ms 
        retlw 0xFF         ;1295rpm | 12 | 3,09ms 
        retlw 0xFF         ;1288rpm | 12 | 3,11ms 
        retlw 0xFF         ;1281rpm | 12 | 3,12ms 
        retlw 0xFF         ;1274rpm | 12 | 3,14ms 
        retlw 0xFF         ;1267rpm | 12 | 3,16ms 
        retlw 0xFF         ;1260rpm | 11 | 3,31ms 
        retlw 0xFF         ;1253rpm | 11 | 3,32ms 
        retlw 0xFF         ;1247rpm | 11 | 3,34ms 
        retlw 0xFF         ;1240rpm | 11 | 3,36ms 
        retlw 0xFF         ;1234rpm | 11 | 3,38ms 
        retlw 0xFF         ;1227rpm | 11 | 3,4ms 
        retlw 0xFF         ;1221rpm | 11 | 3,41ms 
        retlw 0xFF         ;1214rpm | 11 | 3,43ms 
        retlw 0xFF         ;1208rpm | 11 | 3,45ms 
        retlw 0xFF         ;1202rpm | 11 | 3,47ms 
        retlw 0xFF         ;1196rpm | 11 | 3,48ms 
        retlw 0xFF         ;1190rpm | 11 | 3,5ms 
        retlw 0xFF         ;1184rpm | 11 | 3,52ms 
        retlw 0xFF         ;1178rpm | 11 | 3,54ms 
        retlw 0xFF         ;1172rpm | 11 | 3,56ms 
        retlw 0xFF         ;1166rpm | 10 | 3,72ms 
        retlw 0xFF         ;1160rpm | 10 | 3,73ms 
        retlw 0xFF         ;1155rpm | 10 | 3,75ms 
        retlw 0xFF         ;1149rpm | 10 | 3,77ms 
        retlw 0xFF         ;1143rpm | 10 | 3,79ms 
        retlw 0xFF         ;1138rpm | 10 | 3,81ms 
        retlw 0xFF         ;1132rpm | 10 | 3,83ms 
        retlw 0xFF         ;1127rpm | 10 | 3,85ms 
        retlw 0xFF         ;1121rpm | 10 | 3,86ms 
        retlw 0xFF         ;1116rpm | 10 | 3,88ms 
        retlw 0xFF         ;1111rpm | 10 | 3,9ms 
        retlw 0xFF         ;1106rpm | 8 | 4,22ms 
        retlw 0xFF         ;1100rpm | 7 | 4,39ms 
        retlw 0xFF         ;1095rpm | 5 | 4,72ms 
        retlw 0xFF         ;1090rpm | 5 | 4,74ms 
        retlw 0xFF         ;1085rpm | 5 | 4,76ms 
        retlw 0xFF         ;1080rpm | 5 | 4,78ms 
        retlw 0xFF         ;1075rpm | 5 | 4,81ms 
        retlw 0xFF         ;1070rpm | 5 | 4,83ms 
        retlw 0xFF         ;1065rpm | 5 | 4,85ms 
        retlw 0xFF         ;1061rpm | 5 | 4,87ms 
        retlw 0xFF         ;1056rpm | 5 | 4,89ms 
        retlw 0xFF         ;1051rpm | 5 | 4,92ms 
        retlw 0xFF         ;1046rpm | 5 | 4,94ms 
        retlw 0xFF         ;1042rpm | 5 | 4,96ms 
        retlw 0xFF         ;1037rpm | 5 | 4,98ms 
        retlw 0xFF         ;1032rpm | 5 | 5ms 
        retlw 0xFF         ;1028rpm | 5 | 5,03ms 
        retlw 0xFF         ;1023rpm | 5 | 5,05ms 
        retlw 0xFF         ;1019rpm | 5 | 5,07ms 
        retlw 0xFF         ;1015rpm | 5 | 5,09ms 
        retlw 0xFF         ;1010rpm | 5 | 5,11ms 
        retlw 0xFF         ;1006rpm | 5 | 5,14ms 
;				 line 870
;----------------------------------------------------------------------

	ORG 0x300
map2
	BCF    PCLATH,2
	BSF    PCLATH,1
	BSF    PCLATH,0		; select page at b'011' = h'300'
        ADDWF   PCL,f		;line 878
        retlw 0x10
        retlw 0x10
        retlw 0x10
        retlw 0x10
        retlw 0x10
        retlw 0x10
        retlw 0x10
        retlw 0x10
        retlw 0x10
        retlw 0x10
        retlw 0x10
        retlw 0x10
        retlw 0x10
        retlw 0x10
        retlw 0x10
        retlw 0x10
        retlw 0x10
        retlw 0x10
        retlw 0x10
        retlw 0x10
        retlw 0x10
        retlw 0x10
        retlw 0x10
        retlw 0x10
        retlw 0x10
        retlw 0x10
        retlw 0x10
        retlw 0x10
        retlw 0x10
        retlw 0x10
        retlw 0x10
        retlw 0x10
        retlw 0x10
        retlw 0x10
        retlw 0x10
        retlw 0x10
        retlw 0x10
        retlw 0x10
        retlw 0x10
        retlw 0x10
        retlw 0x10
        retlw 0x10
        retlw 0x10
        retlw 0x10
        retlw 0x10
        retlw 0x10
        retlw 0x10
        retlw 0x10
        retlw 0x10
        retlw 0x10
        retlw 0x10
        retlw 0x10
        retlw 0x10
        retlw 0x10
        retlw 0x10
        retlw 0x10
        retlw 0x10
        retlw 0x10
        retlw 0x10
        retlw 0x10
        retlw 0x10
        retlw 0x10
        retlw 0x10
        retlw 0x10
        retlw 0x10
        retlw 0x10
        retlw 0x10
        retlw 0x10
        retlw 0x10
        retlw 0x10
        retlw 0x10
        retlw 0x10
        retlw 0x10
        retlw 0x10
        retlw 0x10
        retlw 0x10
        retlw 0x10
        retlw 0x10
        retlw 0x10
        retlw 0x10
        retlw 0x10
        retlw 0x10
        retlw 0x10
        retlw 0x10
        retlw 0x10
        retlw 0x10
        retlw 0x10
        retlw 0x10
        retlw 0x10
        retlw 0x10
        retlw 0x10
        retlw 0x10
        retlw 0x10
        retlw 0x10
        retlw 0x10
        retlw 0x10
        retlw 0x10
        retlw 0x10
        retlw 0x10
        retlw 0x10
        retlw 0x10
;				 line 1091
; Si votre tableau est  ladresse 0x200, vous devrez mettre 0x02 dans PCLATH.
; Le tableau ne devra pas dpasser 256 lments, et ne devra pas dborder sur la page suivante.
; Emplacements disponibles pour un tableau dune telle taille:
; adresse 0x00. Pas utilisable, car adresse de reset
; adresse 0x100. 1er emplacement utilisable
; adresse 0x200. 2em emplacement utilisable
; adresse 0x300. 3em emplacement utilisable.
; adresse 0x400. 4em et dernier emplacement utilisable.
; Si plus petit tableau, vous pouvez le commencer ailleurs, en veillant bien  ne pas occasionner de dbordement.

;----------------------------------------------------------------------

	org 1c0h		; if the program is loose,
	goto init		; It goes back home.

;---------------------------------------------------------------------
;    END
;---------------------------------------------------------------------
	end
